package org.openxma.dsl.generator.mojo;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.shared.model.fileset.FileSet;
import org.apache.maven.shared.model.fileset.util.FileSetManager;
import org.eclipse.emf.mwe.core.WorkflowContext;

import at.spardat.xma.guidesign.XMAComponent;
import at.spardat.xma.guidesign.util.ValidationError;

/**
 * Maven plugin used to execute the openXMA Pom generator workflow
 *
 * @goal generate-pom
 * @requiresDependencyResolution compile
 * @phase generate-sources
 */
public class PomGeneratorMojo extends AbstractGeneratorMojo {
    // todo-> into generator constants
    private static final String WORKFLOW_POM2_XMA_MWE = "workflow/Pom2Xma.mwe";
    /**
     * @required
     * @parameter
     */
    private FileSet componentFileSet;
    /**
     * @parameter default-value="${basedir}"
     * @required
     * @readonly
     */
    private File projectDirectory;
    /**
     * source folder of the generated xma filed (to be renamed)
     *
     * @parameter default-value="/src/main/java/"
     * @required
     */
    private String sourceDirectory;
    /**
     * @parameter default-value="/src/main/resources/"
     * @required
     * @readonly
     */
    private String resourceFolder;
    /**
     * @parameter default-value="/src/main/resources/guidesign.properties"
     * @required
     * @readonly
     */
    private String guiDesignProperties;

    public void doExecute() throws MojoExecutionException {
        check();
        FileSetManager fileSetManager = new FileSetManager();
        String[] includedFiles = fileSetManager.getIncludedFiles(componentFileSet);
        for (String srcFolderRelativeModelFile : includedFiles) {
            WorkflowContext workFlowContext = runWorkFlow(WORKFLOW_POM2_XMA_MWE,
                    getWorkflowParameterMap(srcFolderRelativeModelFile));
            XMAComponent xmaComponent = (XMAComponent) workFlowContext.get("newModel");
            runXMAGenerator(xmaComponent);
        }
        getLog().info(includedFiles.length + " files transformed.");
    }

    private void runXMAGenerator(XMAComponent xmaComponent) throws MojoExecutionException {
        try {
            @SuppressWarnings("unchecked")
            List<ValidationError> validationErrors = xmaComponent.validate(projectDirectory, guiDesignProperties);
            for (ValidationError validationError : validationErrors) {
                getLog().error(validationError.getMessage());
            }
            if (validationErrors.size() == 1)
                throw new MojoExecutionException(((ValidationError) validationErrors.get(0)).getMessage());
            else if (!validationErrors.isEmpty()) {
                throw new MojoExecutionException("(several validation errors)");
            }
            xmaComponent.generate(projectDirectory, sourceDirectory, resourceFolder, guiDesignProperties);
        } catch (IOException e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    private void check() throws MojoExecutionException {
        if (sourceDirectory == null || sourceDirectory.length() < 1) {
            throw new MojoExecutionException("Missing attribute 'javaSourceFolder'");
        }
        if (componentFileSet == null || componentFileSet.getIncludes() == null
                || componentFileSet.getIncludes().size() == 0) {
            throw new MojoExecutionException(
                    "No files specified. User either the 'file' attribute or a 'fileset' element.");
        }
    }

    private Map<String, String> getWorkflowParameterMap(String relativeModelFile) {
        String projectName = projectDirectory.getName();
        String componentFolder = relativeModelFile.substring(0, relativeModelFile.lastIndexOf(File.separatorChar))
                .replace('\\', '/');
        String fileName = new File(relativeModelFile).getName();
        String componentName = fileName.substring(0, fileName.lastIndexOf('.'));
        String classPathUri = "classpath:/" + relativeModelFile;
        String platformUri = projectDirectory.getParent();
        getLog().info("Projekt-name:           " + projectName);
        getLog().info("Platform-uri:           " + platformUri);
        getLog().info("Component-folder:       " + componentFolder);
        getLog().info("Component-name:         " + componentName);
        getLog().info("Java source folder:     " + sourceDirectory);
        getLog().info("ClassPathUri:           " + classPathUri);
        Map<String, String> properties = new HashMap<String, String>();
        properties.put("platformUri", platformUri);
        properties.put("projectName", projectName);
        properties.put("componentFolder", componentFolder);
        properties.put("componentName", componentName);
        properties.put("javaSourceFolder", sourceDirectory);
        properties.put("classPathUri", classPathUri);
        return properties;

    }

    @Override
    public void putWorkflowParametersToGeneratorProperties(Properties props,
            Map<String, String> workflowParameters) {
    }

}
